@@ -1,6 +1,7 @@
 /* Test file for mpfr_add_ui
 
-Copyright 2000, 2001, 2002, 2003, 2004, 2005 Free Software Foundation.
+Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+Contributed by the Arenaire and Cacao projects, INRIA.
 
 This file is part of the MPFR Library.
 
@@ -16,8 +17,8 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the MPFR Library; see the file COPYING.LIB.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-MA 02111-1307, USA. */
+the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
+MA 02110-1301, USA. */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -31,7 +32,7 @@ check3 (const char *xs, unsigned long y, mp_rnd_t rnd_mode, const char *zs)
 {
   mpfr_t xx, zz;
 
-  mpfr_inits2 (53, xx, zz, NULL);
+  mpfr_inits2 (53, xx, zz, (mpfr_ptr) 0);
   mpfr_set_str1 (xx, xs);
   mpfr_add_ui (zz, xx, y, rnd_mode);
   if (mpfr_cmp_str1(zz, zs) )
@@ -42,7 +43,7 @@ check3 (const char *xs, unsigned long y, mp_rnd_t rnd_mode, const char *zs)
               xs, y, mpfr_print_rnd_mode(rnd_mode));
       exit (1);
   }
-  mpfr_clears (xx, zz, NULL);
+  mpfr_clears (xx, zz, (mpfr_ptr) 0);
 }
 
 static void
@@ -87,6 +88,11 @@ check_nans (void)
   mpfr_clear (y);
 }
 
+#define TEST_FUNCTION mpfr_add_ui
+#define INTEGER_TYPE  unsigned long
+#define RAND_FUNCTION(x) mpfr_random2(x, MPFR_LIMB_SIZE (x), 1)
+#include "tgeneric_ui.c"
+
 int
 main (int argc, char *argv[])
 {
@@ -96,13 +102,14 @@ main (int argc, char *argv[])
 
   special ();
   check3 ("-1.716113812768534e-140", 1271212614, GMP_RNDZ,
-	  "1.27121261399999976e9");
-  check3 ("1.22191250737771397120e+20", 948002822, GMP_RNDN, 
-	  "122191250738719408128.0");
+          "1.27121261399999976e9");
+  check3 ("1.22191250737771397120e+20", 948002822, GMP_RNDN,
+          "122191250738719408128.0");
   check3 ("-6.72658901114033715233e-165", 2000878121, GMP_RNDZ,
-	  "2.0008781209999997615e9");
-  check3 ("-2.0769715792901673e-5", 880524, GMP_RNDN, 
-	  "8.8052399997923023e5");
+          "2.0008781209999997615e9");
+  check3 ("-2.0769715792901673e-5", 880524, GMP_RNDN,
+          "8.8052399997923023e5");
+  test_generic_ui (2, 1000, 100);
 
   tests_end_mpfr ();
   return 0;
