@@ -2,7 +2,8 @@
    mpfr_lessequal_p, mpfr_lessgreater_p, mpfr_equal_p, mpfr_unordered_p
    functions.
 
-Copyright 2003 Free Software Foundation.
+Copyright 2003, 2006, 2007, 2008 Free Software Foundation, Inc.
+Contributed by the Arenaire and Cacao projects, INRIA.
 
 This file is part of the MPFR Library.
 
@@ -18,7 +19,7 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the MPFR Library; see the file COPYING.LIB.  If not, write to
-the Free Software Foundation, Inc., 51 Franklin Place, Fifth Floor, Boston,
+the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301, USA. */
 
 #include <stdio.h>
@@ -32,7 +33,7 @@ cmp_tests (void)
   mpfr_t x, y;
   long i;
 
-  mpfr_inits (x, y, (void *) 0);
+  mpfr_inits (x, y, (mpfr_ptr) 0);
   for (i = 0; i < 80000; i++)
     {
       mp_prec_t precx, precy;
@@ -84,7 +85,7 @@ cmp_tests (void)
           exit (1);
         }
     }
-  mpfr_clears (x, y, (void *) 0);
+  mpfr_clears (x, y, (mpfr_ptr) 0);
 }
 
 static void
@@ -93,7 +94,7 @@ eq_tests (void)
   mpfr_t x, y;
   long i;
 
-  mpfr_inits (x, y, (void *) 0);
+  mpfr_inits (x, y, (mpfr_ptr) 0);
   for (i = 0; i < 20000; i++)
     {
       mp_prec_t precx;
@@ -116,7 +117,7 @@ eq_tests (void)
           exit (1);
         }
     }
-  mpfr_clears (x, y, (void *) 0);
+  mpfr_clears (x, y, (mpfr_ptr) 0);
 }
 
 int
