@@ -1,7 +1,7 @@
 /* Test file for
    mpfr_set_sj, mpfr_set_uj, mpfr_set_sj_2exp and mpfr_set_uj_2exp.
 
-Copyright 2004, 2006, 2007 Free Software Foundation, Inc.
+Copyright 2004, 2006, 2007, 2008 Free Software Foundation, Inc.
 Contributed by the Arenaire and Cacao projects, INRIA.
 
 This file is part of the MPFR Library.
@@ -65,7 +65,7 @@ check_set_uj (mp_prec_t pmin, mp_prec_t pmax, int N)
   int inex1, inex2, n;
   mp_limb_t limb;
 
-  mpfr_inits2 (pmax, x, y, (void *) 0);
+  mpfr_inits2 (pmax, x, y, (mpfr_ptr) 0);
 
   for ( p = pmin ; p < pmax ; p++)
     {
@@ -107,7 +107,7 @@ check_set_uj (mp_prec_t pmin, mp_prec_t pmax, int N)
   if (!MPFR_IS_ZERO (x))
     ERROR ("Setting 0");
 
-  mpfr_clears (x, y, (void *) 0);
+  mpfr_clears (x, y, (mpfr_ptr) 0);
 }
 
 static void
@@ -127,7 +127,7 @@ check_set_uj_2exp (void)
     ERROR("(1024U,-10)");
 
   inex = mpfr_set_uj_2exp (x, 1024, 10, GMP_RNDN);
-  if (inex || mpfr_cmp_ui(x, 1024*1024))
+  if (inex || mpfr_cmp_ui(x, 1024L * 1024L))
     ERROR("(1024U,+10)");
 
   inex = mpfr_set_uj_2exp (x, UINTMAX_MAX, 1000, GMP_RNDN);
