@@ -1,6 +1,7 @@
 /* Test file for mpfr_ui_div.
 
-Copyright 2000, 2001, 2002, 2003, 2004, 2005 Free Software Foundation, Inc.
+Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+Contributed by the Arenaire and Cacao projects, INRIA.
 
 This file is part of the MPFR Library.
 
@@ -16,8 +17,8 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the MPFR Library; see the file COPYING.LIB.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-MA 02111-1307, USA. */
+the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
+MA 02110-1301, USA. */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -30,7 +31,7 @@ check (unsigned long y, const char *xs, mp_rnd_t rnd_mode, const char *zs)
 {
   mpfr_t xx, zz;
 
-  mpfr_inits2 (53, xx, zz, NULL);
+  mpfr_inits2 (53, xx, zz, (mpfr_ptr) 0);
   mpfr_set_str1 (xx, xs);
   mpfr_ui_div (zz, y, xx, rnd_mode);
   if (mpfr_cmp_str1(zz, zs))
@@ -41,7 +42,7 @@ check (unsigned long y, const char *xs, mp_rnd_t rnd_mode, const char *zs)
               y, xs, mpfr_print_rnd_mode (rnd_mode));
       exit (1);
     }
-  mpfr_clears (xx, zz, NULL);
+  mpfr_clears (xx, zz, (mpfr_ptr) 0);
 }
 
 static void
@@ -159,6 +160,12 @@ check_nan (void)
   mpfr_clear (q);
 }
 
+static int
+mpfr_inv (mpfr_t y, mpfr_t x, mp_rnd_t r)
+{
+  return mpfr_ui_div (y, 1, x, r);
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -167,15 +174,18 @@ main (int argc, char *argv[])
   check_nan ();
   check_inexact ();
   check(948002822, "1.22191250737771397120e+20", GMP_RNDN,
-	"7.758352715731357946e-12");
+        "7.758352715731357946e-12");
   check(1976245324, "1.25296395864546893357e+232", GMP_RNDZ,
-	"1.5772563211925444801e-223");
+        "1.5772563211925444801e-223");
   check(740454110, "2.11496253355831863313e+183", GMP_RNDZ,
-	"3.5010270784996976041e-175");
+        "3.5010270784996976041e-175");
   check(1690540942, "1.28278599852446657468e-276", GMP_RNDU,
-	"1.3178666932321966062e285");
+        "1.3178666932321966062e285");
   check(1476599377, "-2.14191393656148625995e+305", GMP_RNDD,
-	"-6.8938315017943889615e-297");
+        "-6.8938315017943889615e-297");
+
+  /* inv is for 1/x */
+  data_check ("data/inv", mpfr_inv, "mpfr_ui_div(1,x)");
 
   tests_end_mpfr ();
   return 0;
