@@ -1,6 +1,6 @@
 /* Test file for mpfr_ui_sub.
 
-Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007 Free Software Foundation, Inc.
+Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 Contributed by the Arenaire and Cacao projects, INRIA.
 
 This file is part of the MPFR Library.
@@ -23,7 +23,6 @@ MA 02110-1301, USA. */
 #include <stdio.h>
 #include <stdlib.h>
 #include <float.h>
-#include <time.h>
 
 #include "mpfr-test.h"
 
@@ -147,7 +146,7 @@ check (unsigned long y, const char *xs, mp_rnd_t rnd_mode, const char *zs)
 {
   mpfr_t xx, zz;
 
-  mpfr_inits2 (53, xx, zz, (void *) 0);
+  mpfr_inits2 (53, xx, zz, (mpfr_ptr) 0);
   mpfr_set_str1 (xx, xs);
   mpfr_ui_sub (zz, y, xx, rnd_mode);
   if (mpfr_cmp_str1 (zz, zs) )
@@ -158,7 +157,7 @@ check (unsigned long y, const char *xs, mp_rnd_t rnd_mode, const char *zs)
               y, xs, mpfr_print_rnd_mode (rnd_mode));
       exit (1);
     }
-  mpfr_clears (xx, zz, (void *) 0);
+  mpfr_clears (xx, zz, (mpfr_ptr) 0);
 }
 
 /* if u = o(x-y), v = o(u-x), w = o(v+y), then x-y = u-w */
@@ -170,7 +169,7 @@ check_two_sum (mp_prec_t p)
   mp_rnd_t rnd;
   int inexact;
 
-  mpfr_inits2 (p, y, u, v, w, (void *) 0);
+  mpfr_inits2 (p, y, u, v, w, (mpfr_ptr) 0);
   do
     {
       x = randlimb ();
@@ -196,7 +195,7 @@ check_two_sum (mp_prec_t p)
       printf ("inexact = %d\n", inexact);
       exit (1);
     }
-  mpfr_clears (y, u, v, w, (void *) 0);
+  mpfr_clears (y, u, v, w, (mpfr_ptr) 0);
 }
 
 static void
